"app/database/*"

from textwrap import dedent

APP_REPO_INIT_CONTENT_FOR_SQL = dedent("""
from .hero import HeroRepoDeps, HeroRepository

__all__ = ["HeroRepoDeps", "HeroRepository"]
""")

APP_REPO_HERO_CONTENT_FOR_SQL = dedent("""
from typing import Annotated

from fastapi import Depends
from sqlalchemy import select
from sqlalchemy.exc import IntegrityError as SAIntegrityError
from sqlalchemy.ext.asyncio import AsyncSession

from app.core.errors import (
    DatabaseError,
    DuplicateKeyError,
    NotFoundError,
)
from app.database import SessionDeps
from app.models import Hero


class HeroRepository:
    def __init__(self, session: AsyncSession):
        self.session = session

    async def get_hero_by_id(self, hero_id) -> Hero:
        result = await self.session.execute(select(Hero).where(Hero.id == hero_id))
        hero = result.scalar_one_or_none()

        if not hero:
            raise NotFoundError("Hero not found")

        return hero

    async def create_hero(self, name: str) -> Hero:
        hero = Hero(name=name)
        self.session.add(hero)

        try:
            await self.session.commit()
            await self.session.refresh(hero)
            return hero

        except SAIntegrityError as e:
            await self.session.rollback()
            raise DuplicateKeyError("Hero with this name already exists") from e

        except Exception as e:
            await self.session.rollback()
            raise DatabaseError("Failed to create hero") from e

    async def update_hero(self, hero_id, name: str | None = None) -> Hero:
        hero = await self.get_hero_by_id(hero_id)

        if name is not None:
            hero.name = name

        try:
            await self.session.commit()
            await self.session.refresh(hero)
            return hero

        except SAIntegrityError as e:
            await self.session.rollback()
            raise DuplicateKeyError("Hero with this name already exists") from e

        except Exception as e:
            await self.session.rollback()
            raise DatabaseError("Failed to update hero") from e

    async def delete_hero(self, hero_id) -> Hero:
        hero = await self.get_hero_by_id(hero_id)

        try:
            await self.session.delete(hero)
            await self.session.commit()
            return hero

        except Exception as e:
            await self.session.rollback()
            raise DatabaseError("Failed to delete hero") from e


def get_hero_repo(
    session: SessionDeps,
):
    return HeroRepository(session)


HeroRepoDeps = Annotated[HeroRepository, Depends(get_hero_repo)]
""")



"""app/core/*"""

from textwrap import dedent

APP_CORE_INIT_CONTENT = dedent(
    """
from .config import settings
from .lifespan import lifespan
from .response_helper import success_response, error_response

__all__ = ["settings", "lifespan", "success_response", "error_response"]
"""
)

APP_CORE_CONFIG_CONTENT = dedent("""
from pydantic_settings import BaseSettings, SettingsConfigDict
from dotenv import load_dotenv
from pydantic import Field
from functools import lru_cache
from typing import Literal

load_dotenv()


class Settings(BaseSettings):
    '''
    Settings for the application
    '''

    model_config = SettingsConfigDict(
        env_file=".env", env_file_encoding="utf-8", extra="ignore"
    )

    TITLE: str = Field(
        description="The title of the application", validation_alias="TITLE"
    )

    DESCRIPTION: str = Field(
        description="The description of the application",
        validation_alias="DESCRIPTION",
    )

    VERSION: str = Field(
        description="The version of the application", validation_alias="VERSION"
    )

    DEBUG: bool = Field(
        description="Whether the application is in debug mode",
        validation_alias="DEBUG",
    )

    PORT: int = Field(
        description="The port to run the application on", validation_alias="PORT"
    )

    HOST: str = Field(
        description="The host to run the application on", validation_alias="HOST"
    )

    DB_USER: str = Field(..., description="Database user name", validation_alias="DB_USER")

    DB_PASS: str = Field(..., description="Database password", validation_alias="DB_PASS")

    DB_HOST: str = Field(..., description="Database Host", validation_alias="DB_HOST")

    DB_PORT: str = Field(..., description="Database port", validation_alias="DB_PORT")

    DB_NAME: str = Field(..., description="Database Name", validation_alias="DB_NAME")

    DB_TYPE: Literal["postgres", "mysql", "sqlite", "mongodb"] = Field(
        "postgres", description="Which database to use for the project", validation_alias="DB_TYPE"
    )

    @property
    def db_url(self) -> str:
        if self.DB_TYPE == "postgres":
            return f"postgresql+asyncpg://{self.DB_USER}:{self.DB_PASS}@{self.DB_HOST}:{self.DB_PORT}/{self.DB_NAME}"


@lru_cache(maxsize=1)
def get_settings() -> Settings:
    '''
    Get the settings
    '''
    return Settings()


settings = get_settings()
""")

APP_CORE_LIFESPAN_CONTENT = dedent("""
from fastapi import FastAPI
from contextlib import asynccontextmanager
from app.database.session import engine


@asynccontextmanager
async def lifespan(app: FastAPI):
    yield
    await engine.dispose()
""")

APP_CORE_API_RESPONSE_HELPER_CONTENT = dedent(
    """
from fastapi import status
from fastapi.responses import JSONResponse
from typing import Any
from app.schemas.common import APIResponse


def api_response(
    *,
    success: bool,
    message: str | None,
    data: Any = None,
    status_code: int = status.HTTP_200_OK,
    error_code: str | None = None,
    stack_trace: str | None = None,
) -> JSONResponse:
    payload = APIResponse(
        success=success,
        message=message,
        data=data,
        error_code=error_code,
        stack_trace=stack_trace,
    )

    return JSONResponse(
        status_code=status_code,
        content=payload.model_dump(exclude_none=True),
    )


def success_response(
    message: str | None = None,
    data: Any = None,
    status_code: int = status.HTTP_200_OK,
):
    return api_response(
        success=True,
        message=message,
        data=data,
        status_code=status_code,
    )


def error_response(
    message: str,
    status_code: int = status.HTTP_400_BAD_REQUEST,
    error_code: str | None = None,
    data: Any = None,
    stack_trace: str | None = None,
):
    return api_response(
        success=False,
        message=message,
        status_code=status_code,
        error_code=error_code,
        data=data,
        stack_trace=stack_trace,
    )
"""
)

APP_CORE_ERRORS_CONTENT = dedent('''
from typing import Any, Optional
from fastapi import status
import traceback


class AppError(Exception):
    status_code: int = status.HTTP_400_BAD_REQUEST
    message: str = "Application error"
    error_code: str = "APP_ERROR"
    data: Any = None
    stack_trace: Optional[str] = None

    def __init__(
        self,
        message: Optional[str] = None,
        *,
        data: Any = None,
        error_code: Optional[str] = None,
        capture_trace: bool = True,
    ):
        if message:
            self.message = message
        if error_code:
            self.error_code = error_code
        self.data = data
        if capture_trace:
            self.stack_trace = traceback.format_exc()
        super().__init__(self.message)


"""Validation & Request errors (replaces 422)"""


class ValidationError(AppError):
    status_code = status.HTTP_400_BAD_REQUEST
    error_code = "VALIDATION_ERROR"
    message = "Invalid request data"


"""Authentication & Authorization"""


class UnauthorizedError(AppError):
    status_code = status.HTTP_401_UNAUTHORIZED
    error_code = "UNAUTHORIZED"
    message = "Authentication required"


class ForbiddenError(AppError):
    status_code = status.HTTP_403_FORBIDDEN
    error_code = "FORBIDDEN"
    message = "You do not have permission"


"""Resource errors"""


class NotFoundError(AppError):
    status_code = status.HTTP_404_NOT_FOUND
    error_code = "NOT_FOUND"
    message = "Resource not found"


"""Conflict / Integrity errors"""


class IntegrityError(AppError):
    status_code = status.HTTP_409_CONFLICT
    error_code = "INTEGRITY_ERROR"
    message = "Integrity constraint violated"


class DuplicateKeyError(IntegrityError):
    error_code = "DUPLICATE_KEY"
    message = "Duplicate key exists"


"""Mongo-specific"""


class MongoDuplicateKeyError(AppError):
    status_code = status.HTTP_409_CONFLICT
    error_code = "MONGO_DUPLICATE_KEY"
    message = "Duplicate document exists"


"""
Business rule errors
    Examples:
        - Insufficient balance
        - Invalid state transition
        - Feature disabled
"""


class BusinessRuleError(AppError):
    status_code = status.HTTP_400_BAD_REQUEST
    error_code = "BUSINESS_RULE_VIOLATION"
    message = "Business rule violated"


"""External service errors"""


class ExternalServiceError(AppError):
    status_code = status.HTTP_502_BAD_GATEWAY
    error_code = "EXTERNAL_SERVICE_ERROR"
    message = "External service failed"


"""Rate limiting"""


class RateLimitExceededError(AppError):
    status_code = status.HTTP_429_TOO_MANY_REQUESTS
    error_code = "RATE_LIMIT_EXCEEDED"
    message = "Too many requests"


"""Infrastructure / system errors"""


class DatabaseError(AppError):
    status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
    error_code = "DATABASE_ERROR"
    message = "Database operation failed"


class CacheError(AppError):
    status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
    error_code = "CACHE_ERROR"
    message = "Cache operation failed"


"""Fallback"""


class InternalServerError(AppError):
    status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
    error_code = "INTERNAL_SERVER_ERROR"
    message = "Something went wrong"
''')

APP_CORE_EXCEPTIONS_CONTENT = dedent(
    """
from fastapi import FastAPI, Request, status
from fastapi.exceptions import RequestValidationError
from starlette.exceptions import HTTPException as StarletteHTTPException

from .response_helper import error_response
from .errors import AppError
from .config import settings


def register_exception_handlers(app: FastAPI):
    @app.exception_handler(AppError)
    async def app_error_handler(request: Request, exc: AppError):
        stack_trace = exc.stack_trace if settings.DEBUG else None
        return error_response(
            message=exc.message,
            status_code=exc.status_code,
            error_code=exc.error_code,
            data=exc.data,
            stack_trace=stack_trace,
        )

    # Replace FastAPI 422
    @app.exception_handler(RequestValidationError)
    async def validation_error_handler(request: Request, exc: RequestValidationError):
        return error_response(
            message=exc.errors()[0]["msg"],
            status_code=status.HTTP_422_UNPROCESSABLE_CONTENT,
            error_code="VALIDATION_ERROR",
            data=exc.errors(),
        )

    # Catch raw HTTPExceptions
    @app.exception_handler(StarletteHTTPException)
    async def http_exception_handler(request: Request, exc: StarletteHTTPException):
        return error_response(
            message=exc.detail,
            status_code=exc.status_code,
            error_code="HTTP_EXCEPTION",
        )

    # Absolute fallback
    @app.exception_handler(Exception)
    async def unhandled_exception_handler(request: Request, exc: Exception):
        return error_response(
            message="Internal server error",
            status_code=500,
            error_code="UNHANDLED_EXCEPTION",
        )
"""
)

APP_CORE_API_RESPONSES_CONTENT = dedent(
    """
from fastapi import status
from app.schemas.common import SuccessResponseSchema, ErrorResponseSchema


# -------- Error responses --------

VALIDATION_ERROR_RESPONSES = {
    status.HTTP_400_BAD_REQUEST: {
        "description": "Domain validation error",
        "model": ErrorResponseSchema,
    },
    status.HTTP_422_UNPROCESSABLE_ENTITY: {
        "description": "Request validation error",
        "model": ErrorResponseSchema,
    },
}

AUTH_ERROR_RESPONSES = {
    status.HTTP_401_UNAUTHORIZED: {
        "description": "Authentication required or failed",
        "model": ErrorResponseSchema,
    },
    status.HTTP_403_FORBIDDEN: {
        "description": "Permission denied",
        "model": ErrorResponseSchema,
    },
}

NOT_FOUND_RESPONSES = {
    status.HTTP_404_NOT_FOUND: {
        "description": "Resource not found",
        "model": ErrorResponseSchema,
    },
}

CONFLICT_RESPONSES = {
    status.HTTP_409_CONFLICT: {
        "description": "Resource already exists",
        "model": ErrorResponseSchema,
    },
}

SERVER_ERROR_RESPONSES = {
    status.HTTP_500_INTERNAL_SERVER_ERROR: {
        "description": "Internal server error",
        "model": ErrorResponseSchema,
    },
}


# -------- Success responses --------

SUCCESS_200_RESPONSE = {
    status.HTTP_200_OK: {
        "description": "Successful operation",
        "model": SuccessResponseSchema,
    }
}

SUCCESS_201_RESPONSE = {
    status.HTTP_201_CREATED: {
        "description": "Resource created successfully",
        "model": SuccessResponseSchema,
    }
}

SUCCESS_202_RESPONSE = {
    status.HTTP_202_ACCEPTED: {
        "description": "Request accepted",
        "model": SuccessResponseSchema,
    }
}

SUCCESS_204_RESPONSE = {
    status.HTTP_204_NO_CONTENT: {
        "description": "No content",
    }
}
"""
)



"""app/api/*"""

from textwrap import dedent


APP_API_V1_INIT_CONTENT = dedent("""
from .hero import route as hero_router
from fastapi import APIRouter

router = APIRouter(prefix="/v1")


router.include_router(hero_router)

__all__ = ["router"]
""")


APP_API_V1_HERO_CONTENT = dedent("""
from fastapi import APIRouter, status
from uuid import UUID as uuid
from app.core import success_response
from app.services import HeroServiceDeps
from app.schemas.hero import CreateHeroSchema, HeroResponse, UpdateHeroSchema
from app.core.responses import (
    SUCCESS_201_RESPONSE,
    CONFLICT_RESPONSES,
    VALIDATION_ERROR_RESPONSES,
    SERVER_ERROR_RESPONSES,
    SUCCESS_200_RESPONSE,
    NOT_FOUND_RESPONSES,
    SUCCESS_204_RESPONSE,
)

route = APIRouter(prefix="/heroes", tags=["Heroes"])


@route.post(
    "/",
    summary="Create A Hero",
    description="Create a new hero with the given name.",
    status_code=status.HTTP_201_CREATED,
    response_model=HeroResponse,
    responses={
        **SUCCESS_201_RESPONSE,
        **CONFLICT_RESPONSES,
        **VALIDATION_ERROR_RESPONSES,
        **SERVER_ERROR_RESPONSES,
    },
)
async def create_hero(payload: CreateHeroSchema, service: HeroServiceDeps):
    hero = await service.create_hero(name=payload.name)
    return success_response(
        message="Hero created successfully",
        data=hero,
        status_code=status.HTTP_201_CREATED,
    )


@route.get(
    "/{hero_id}",
    summary="Get A Hero Details",
    description="Get a Hero By ID.",
    status_code=status.HTTP_200_OK,
    response_model=HeroResponse,
    responses={
        **SUCCESS_200_RESPONSE,
        **VALIDATION_ERROR_RESPONSES,
        **NOT_FOUND_RESPONSES,
        **SERVER_ERROR_RESPONSES,
    },
)
async def get_hero(hero_id: uuid, service: HeroServiceDeps):
    hero = await service.get_hero(hero_id)
    return success_response(
        message="Hero retrieved successfully",
        data=hero,
        status_code=status.HTTP_200_OK,
    )


@route.put(
    "/{hero_id}",
    summary="Update A Hero Details",
    description="Update hero name by hero ID",
    status_code=status.HTTP_200_OK,
    response_model=HeroResponse,
    responses={
        **SUCCESS_200_RESPONSE,
        **VALIDATION_ERROR_RESPONSES,
        **NOT_FOUND_RESPONSES,
        **CONFLICT_RESPONSES,
        **SERVER_ERROR_RESPONSES,
    },
)
async def update_hero(payload: UpdateHeroSchema, service: HeroServiceDeps):
    hero = await service.update_hero(hero_id=payload.id, name=payload.name)
    return success_response(
        message="Hero updated successfully",
        data=hero,
        status_code=status.HTTP_200_OK,
    )


@route.delete(
    "/{hero_id}",
    summary="Delete A Hero",
    description="Delete hero by hero ID",
    status_code=status.HTTP_204_NO_CONTENT,
    response_model=None,
    responses={
        **SUCCESS_204_RESPONSE,
        **VALIDATION_ERROR_RESPONSES,
        **NOT_FOUND_RESPONSES,
        **SERVER_ERROR_RESPONSES,
    },
)
async def delete_hero(hero_id: uuid, service: HeroServiceDeps):
    await service.delete_hero(hero_id)
    return
""")



"""app/models/*"""

from textwrap import dedent

APP_MODELS_INIT_CONTENT = dedent(
    """
from .hero import Hero

__all__ = ["Hero"]
"""
)

APP_MODELS_HERO_CONTENT = dedent(
    """
from .common import BaseModel
from sqlmodel import Field


class Hero(BaseModel, table=True):
    name: str = Field(index=True, default="", description="The name of the hero", unique=True)

    def to_dict(self) -> dict:
        return {
            "id": str(self.id),
            "name": self.name,
            "created_at": self.created_at.strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
            "updated_at": self.updated_at.strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
        }
"""
)

APP_MODELS_COMMON_CONTENT = dedent("""
from sqlmodel import Field, SQLModel
from sqlalchemy import Column, DateTime
from datetime import datetime, timezone
import uuid


class BaseModel(SQLModel):
    id: uuid.UUID = Field(default_factory=uuid.uuid4, primary_key=True)

    created_at: datetime = Field(
        sa_column=Column(
            DateTime(timezone=True),
            nullable=False,
        ),
        default_factory=lambda: datetime.now(timezone.utc),
    )

    updated_at: datetime = Field(
        sa_column=Column(
            DateTime(timezone=True),
            nullable=False,
        ),
        default_factory=lambda: datetime.now(timezone.utc),
    )
""")



"""app/models/*"""

from textwrap import dedent

APP_SCHEMA_COMMON_CONTENT = dedent(
    """
from typing import Generic, TypeVar, Optional, Any
from pydantic import BaseModel

T = TypeVar("T")


class APIResponse(BaseModel, Generic[T]):
    success: bool
    message: str | None
    data: Optional[T] = None
    error_code: Optional[str] = None
    stack_trace: Optional[str] = None


class SuccessResponseSchema(BaseModel, Generic[T]):
    success: bool = True
    message: str
    data: Optional[T] = None


class ErrorResponseSchema(BaseModel):
    success: bool = False
    message: str
    data: Optional[Any] = None
    error_code: Optional[str] = None
"""
)

APP_SCHEMA_HERO_CONTENT = dedent(
    """
from pydantic import BaseModel, Field
from uuid import UUID
from datetime import datetime
from app.schemas.common import SuccessResponseSchema


# ---------- REQUEST SCHEMAS ----------


class CreateHeroSchema(BaseModel):
    name: str = Field(
        ..., min_length=1, max_length=100, description="The name of the hero."
    )


class UpdateHeroSchema(BaseModel):
    id: UUID = Field(..., description="The unique identifier of the hero.")
    name: str | None = Field(
        None, min_length=1, max_length=100, description="The new name of the hero."
    )


# ---------- RESPONSE SCHEMAS ----------
class HeroSchema(BaseModel):
    id: UUID = Field(..., description="The unique identifier of the hero.")
    name: str = Field(..., description="The name of the hero.")
    created_at: datetime = Field(..., description="The creation timestamp of the hero.")
    updated_at: datetime = Field(
        ..., description="The last update timestamp of the hero."
    )


class HeroResponse(SuccessResponseSchema):
    data: HeroSchema
"""
)



from .app_api import APP_API_COMMON_CONTENT, APP_API_INIT_CONTENT
from .app_api_v1 import APP_API_V1_HERO_CONTENT, APP_API_V1_INIT_CONTENT
from .app_core import (
    APP_CORE_CONFIG_CONTENT,
    APP_CORE_INIT_CONTENT,
    APP_CORE_LIFESPAN_CONTENT,
    APP_CORE_ERRORS_CONTENT,
    APP_CORE_EXCEPTIONS_CONTENT,
    APP_CORE_API_RESPONSE_HELPER_CONTENT,
    APP_CORE_API_RESPONSES_CONTENT,
)
from .app_database import (
    APP_DATABASE_INIT_CONTENT_FOR_SQL,
    APP_DATABASE_SESSION_CONTENT_FOR_SQL,
)
from .app_main import APP_MAIN_CONTENT
from .app_model import (
    APP_MODELS_COMMON_CONTENT,
    APP_MODELS_HERO_CONTENT,
    APP_MODELS_INIT_CONTENT,
)
from .app_repository import APP_REPO_HERO_CONTENT_FOR_SQL, APP_REPO_INIT_CONTENT_FOR_SQL
from .app_services import APP_SERVICE_INIT_CONTENT, APP_SERVICE_HERO_CONTENT
from .app_templates import APP_TEMPLATES_INDEX_CONTENT
from .env_file import ENV_FILE_CONTENT
from .gitignore_file import GIT_IGNORE_FILE_CONTENT
from .migrations_env_file import MIGRATIONS_ENV_FILE_CONTENT
from .run_file import RUN_FILE_CONTENT
from .app_schema import APP_SCHEMA_COMMON_CONTENT, APP_SCHEMA_HERO_CONTENT

__all__ = [
    # main
    APP_MAIN_CONTENT,
    # API
    APP_API_INIT_CONTENT,
    APP_API_COMMON_CONTENT,
    # Core
    APP_CORE_INIT_CONTENT,
    APP_CORE_CONFIG_CONTENT,
    APP_CORE_LIFESPAN_CONTENT,
    APP_CORE_ERRORS_CONTENT,
    APP_CORE_API_RESPONSES_CONTENT,
    APP_CORE_EXCEPTIONS_CONTENT,
    APP_CORE_API_RESPONSE_HELPER_CONTENT,
    # Database
    APP_DATABASE_SESSION_CONTENT_FOR_SQL,
    # Model
    APP_MODELS_COMMON_CONTENT,
    # Templates
    APP_TEMPLATES_INDEX_CONTENT,
    # .env File
    ENV_FILE_CONTENT,
    # .gitignore File
    GIT_IGNORE_FILE_CONTENT,
    # alemmbic Migrations env file
    MIGRATIONS_ENV_FILE_CONTENT,
    # API v1
    APP_API_V1_INIT_CONTENT,
    APP_API_V1_HERO_CONTENT,
    # Database init
    APP_DATABASE_INIT_CONTENT_FOR_SQL,
    APP_DATABASE_INIT_CONTENT_FOR_SQL,
    # Model init
    APP_MODELS_HERO_CONTENT,
    APP_MODELS_INIT_CONTENT,
    # Repository
    APP_REPO_INIT_CONTENT_FOR_SQL,
    APP_REPO_HERO_CONTENT_FOR_SQL,
    # Services
    APP_SERVICE_INIT_CONTENT,
    APP_SERVICE_HERO_CONTENT,
    # run.py
    RUN_FILE_CONTENT,
    # Schemas
    APP_SCHEMA_COMMON_CONTENT,
    APP_SCHEMA_HERO_CONTENT,
]



"""run.py"""

from textwrap import dedent

RUN_FILE_CONTENT = dedent(
    """
import uvicorn
from app.core.config import settings


def main():
    uvicorn.run(
        "app.main:app", host=settings.HOST, port=settings.PORT, reload=settings.DEBUG
    )


if __name__ == "__main__":
    main()
"""
)



"app/main.py"

from textwrap import dedent

APP_MAIN_CONTENT = dedent('''
from fastapi import FastAPI
from app.core import settings, lifespan
from app.api import common_router, v1_router
from app.core.execptions import register_exception_handlers


def create_app() -> FastAPI:
    """
    Create a fastapi app
    """
    app = FastAPI(
        docs_url=None,
        redoc_url=None,
        title=settings.TITLE,
        description=settings.DESCRIPTION,
        version=settings.VERSION,
        lifespan=lifespan,
    )

    app.include_router(common_router)
    app.include_router(v1_router)

    register_exception_handlers(app)

    return app


app = create_app()
''')



from builders_hut.setups import BaseSetup
from builders_hut.utils import make_file


class SetupFiles(BaseSetup):
    """
    Create all the required files for the project.
    """

    FILES_TO_CREATE = [
        # Main application file
        "app/main.py",
        # api files
        "app/api/__init__.py",
        "app/api/common.py",
        "app/api/v1/__init__.py",
        "app/api/v1/hero.py",
        # Core configuration and logger files
        "app/core/__init__.py",
        "app/core/config.py",
        "app/core/errors.py",
        "app/core/execptions.py",
        "app/core/lifespan.py",
        "app/core/logger.py",
        "app/core/response_helper.py",
        "app/core/responses.py",
        # database files
        "app/database/__init__.py",
        "app/database/session.py",
        # Model files
        "app/models/__init__.py",
        "app/models/common.py",
        "app/models/hero.py",
        # Repository files
        "app/repositories/__init__.py",
        "app/repositories/hero.py",
        # Schema files
        "app/schemas/__init__.py",
        "app/schemas/hero.py",
        "app/schemas/common.py",
        # Service files
        "app/services/__init__.py",
        "app/services/hero.py",
        # templates files
        "app/templates/index.html",
        # Utils files
        "app/utils/__init__.py",
        # Workers files
        "app/workers/__init__.py",
        # Test initialization file
        "tests/__init__.py",
        # Configuration
        ".env",
        ".gitignore",
        "requirements.txt",
        "requirements_dev.txt",
        "run.py",
    ]

    def create(self):
        for file_path in self.FILES_TO_CREATE:
            full_path = self.location / file_path
            make_file(full_path)



"""app/models/*"""

from textwrap import dedent

APP_SERVICE_INIT_CONTENT = dedent(
    """
from .hero import HeroServiceDeps

__all__ = ["HeroServiceDeps"]
"""
)

APP_SERVICE_HERO_CONTENT = dedent(
    """
from typing import Annotated

from fastapi import Depends

from app.core.errors import ValidationError
from app.repositories import HeroRepoDeps, HeroRepository


class HeroService:
    def __init__(self, repo: HeroRepository):
        self.repo = repo

    async def get_hero(self, hero_id):
        hero = await self.repo.get_hero_by_id(hero_id)
        return hero.to_dict()

    async def create_hero(self, name: str):
        if not name or not name.strip():
            raise ValidationError(
                message="Hero name cannot be empty",
                data={"field": "name"},
            )

        hero = await self.repo.create_hero(name=name.strip())
        return hero.to_dict()

    async def update_hero(self, hero_id, name: str | None = None):
        if name is not None and not name.strip():
            raise ValidationError(
                message="Hero name cannot be empty",
                data={"field": "name"},
            )

        hero = await self.repo.update_hero(hero_id, name=name.strip() if name else None)
        return hero.to_dict()

    async def delete_hero(self, hero_id):
        return await self.repo.delete_hero(hero_id)


def get_hero_service(
    repo: HeroRepoDeps,
):
    return HeroService(repo)


HeroServiceDeps = Annotated[HeroService, Depends(get_hero_service)]
"""
)



from pathlib import Path
from builders_hut.setups.base_setup import BaseSetup
import platform
import tomlkit
import subprocess


def write_pyproject(
    path: str | Path = "pyproject.toml",
    *,
    name: str,
    version: str = "0.1.0",
    description: str = "",
    python: str = ">=3.13",
    dependencies: list[str] | None = None,
    dev_dependencies: list[str] | None = None,
) -> None:
    """
    Completely write a pyproject.toml file.

    This function OVERWRITES any existing file.
    Intended for scaffolding / project initialization.
    """

    path = Path(path)
    doc = tomlkit.document()

    # ------------------
    # [project]
    # ------------------
    project = tomlkit.table()
    project.add("name", name)
    project.add("version", version)

    if description:
        project.add("description", description)

    project.add("requires-python", python)

    project.add(
        "dependencies",
        dependencies or [],
    )

    if dev_dependencies:
        optional = tomlkit.table()
        optional.add("dev", dev_dependencies)
        project.add("optional-dependencies", optional)

    # ------------------
    # [project.scripts]
    # ------------------
    scripts = tomlkit.table()
    scripts.add("run_dev_server", "app.scripts.dev:main")
    scripts.add("run_prod_server", "app.scripts.prod:main")
    project.add("scripts", scripts)

    doc.add("project", project)

    # Write file
    path.write_text(tomlkit.dumps(doc), encoding="utf-8")


def setup_project(location: Path, setup_cls: type[BaseSetup], **config) -> None:
    """
    Take BaseSetup and make the project
    """
    setup = setup_cls(location)
    setup.configure(**config)
    setup.create()


def get_platform():
    """check the operating system"""
    system = platform.system()
    return system.lower()


def make_folder(loc: Path):
    """make a folder in the given path"""
    loc.mkdir(exist_ok=True, parents=True)


def make_file(file: Path):
    """make a file in the given location"""
    file.touch(exist_ok=True)


def write_file(path: Path, content: str) -> None:
    """write data to a file"""
    if not path.exists():
        raise FileNotFoundError(f"{path} does not exist")

    path.write_text(content, encoding="utf-8")


def run_subprocess(location: Path, command: str):
    """
    Run subprocess command
    """
    subprocess.run(
        command,
        cwd=location,
        shell=True,
        check=True,
        stderr=subprocess.DEVNULL,
        stdout=subprocess.DEVNULL,
    )


def get_python_file():
    """use the python file in the venv"""
    platform = get_platform()

    python_file = (
        ".venv/bin/python -m"
        if platform == "linux"
        else ".venv\\Scripts\\python.exe -m"
    )

    return python_file




from builders_hut.setups import BaseSetup
from builders_hut.utils import write_file, run_subprocess, get_python_file
from typing import Literal

PACKAGES = [
    "fastapi",
    "python-dotenv",
    "email-validator",
    "tzdata",
    "pydantic-settings",
    "scalar-fastapi",
    "uvicorn",
    "jinja2",
]

DEV_PACKAGES = ["pytest"]


DB_SQL_PACKAGES = {
    "postgres": ["sqlmodel", "asyncpg", "psycopg2-binary"],
    "mysql": ["sqlmodel", "aiomysql", "pymysql"],
    "sqlite": ["sqlmodel", "aiosqlite"],
}

SQL_COMMON_PACKAGE = ["alembic"]

DEV_PACKAGES_EXTENDED = ["-r requirements.txt"]


class SetupEnv(BaseSetup):
    """Create Env and Install Base Packages"""

    def create(self):
        try:
            try:
                if self.database_type == "sql":
                    PACKAGES.extend(SQL_COMMON_PACKAGE)
                    PACKAGES.extend(DB_SQL_PACKAGES[self.database_provider])

                write_file(self.location / "requirements.txt", "\n".join(PACKAGES))

                DEV_PACKAGES_EXTENDED.extend(DEV_PACKAGES)
                write_file(
                    self.location / "requirements_dev.txt",
                    "\n".join(DEV_PACKAGES_EXTENDED),
                )

            except Exception as e:
                print(e)
                raise RuntimeError("Could not write to requirements file")

            try:
                run_subprocess(self.location, "python -m venv .venv")

            except Exception:
                raise RuntimeError("Could not create virtual env for project")

            command = "pip install -r requirements_dev.txt"
            python_file = get_python_file()

            full_command = f"{python_file} {command}"

            try:
                run_subprocess(self.location, full_command)
            except Exception:
                raise RuntimeError("Could not install packages")

        except Exception as e:
            raise RuntimeError(f"Failed to create environment: {str(e)}")

    def configure(
        self,
        name: str,
        description: str,
        version: str,
        database_provider: Literal["postgres", "mysql", "sqlite", "mongodb"],
        database_type: Literal["sql", "nosql"],
        **kwargs,
    ):
        self.name = name
        self.description = description
        self.version = version
        self.database_provider = database_provider
        self.database_type = database_type



from pathlib import Path

from builders_hut.setups import file_contents

FILES_TO_WRITE: dict[Path, str] = {
    # Main
    Path("app/main.py"): file_contents.APP_MAIN_CONTENT,
    # API
    Path("app/api/__init__.py"): file_contents.APP_API_INIT_CONTENT,
    Path("app/api/common.py"): file_contents.APP_API_COMMON_CONTENT,
    Path("app/api/v1/__init__.py"): file_contents.APP_API_V1_INIT_CONTENT,
    Path("app/api/v1/hero.py"): file_contents.APP_API_V1_HERO_CONTENT,
    # Core
    Path("app/core/__init__.py"): file_contents.APP_CORE_INIT_CONTENT,
    Path("app/core/config.py"): file_contents.APP_CORE_CONFIG_CONTENT,
    Path("app/core/errors.py"): file_contents.APP_CORE_ERRORS_CONTENT,
    Path("app/core/execptions.py"): file_contents.APP_CORE_EXCEPTIONS_CONTENT,
    Path("app/core/lifespan.py"): file_contents.APP_CORE_LIFESPAN_CONTENT,
    Path("app/core/responses.py"): file_contents.APP_CORE_API_RESPONSES_CONTENT,
    Path(
        "app/core/response_helper.py"
    ): file_contents.APP_CORE_API_RESPONSE_HELPER_CONTENT,
    # Models
    Path("app/models/__init__.py"): file_contents.APP_MODELS_INIT_CONTENT,
    Path("app/models/hero.py"): file_contents.APP_MODELS_HERO_CONTENT,
    Path("app/models/common.py"): file_contents.APP_MODELS_COMMON_CONTENT,
    # Repositories
    Path("app/repositories/__init__.py"): file_contents.APP_REPO_INIT_CONTENT_FOR_SQL,
    Path("app/repositories/hero.py"): file_contents.APP_REPO_HERO_CONTENT_FOR_SQL,
    # Services
    Path("app/services/__init__.py"): file_contents.APP_SERVICE_INIT_CONTENT,
    Path("app/services/hero.py"): file_contents.APP_SERVICE_HERO_CONTENT,
    # Templates
    Path("app/templates/index.html"): file_contents.APP_TEMPLATES_INDEX_CONTENT,
    # .env Files
    Path(".env"): file_contents.ENV_FILE_CONTENT,
    # .gitignore File
    Path(".gitignore"): file_contents.GIT_IGNORE_FILE_CONTENT,
    # run.py
    Path("run.py"): file_contents.RUN_FILE_CONTENT,
    # Schemas
    Path("app/schemas/hero.py"): file_contents.APP_SCHEMA_HERO_CONTENT,
    Path("app/schemas/common.py"): file_contents.APP_SCHEMA_COMMON_CONTENT,
}



from builders_hut.setups import BaseSetup
from builders_hut.utils import run_subprocess


class SetupGithub(BaseSetup):
    """
    Setup github for this project
    """

    def create(self):
        try:
            run_subprocess(self.location, "git init")
        except Exception:
            raise RuntimeError("Could Not Initialize Git")
